--------------------------------------------------------------------------------
-- EETAC - CSD - P8 - http://digsys.upc.edu
--------------------------------------------------------------------------------
-- Example of a datapath for a programmable timer. Hierarchical structural 
-- schematic basically composed of arithmetic, logic and memory circuits:
-- From the skethc in https://digsys.upc.edu/csd/exams/D2_21/D2_21.html
--------------------------------------------------------------------------------

LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
	
ENTITY Timer_Datapath	IS  
   Port (
	CLK, CD      		: IN	STD_LOGIC;
	TCLK			: IN	STD_LOGIC; -- The time base for the counter
	LD_time, LD_Counter   	: IN	STD_LOGIC;  
	CNT_enable		: IN	STD_LOGIC;  
	T1, T2, T3, T4 		: IN	STD_LOGIC_VECTOR (7 downto 0);  
	Sel_Time		: IN	STD_LOGIC_VECTOR (1 downto 0);  
	Terminal_Count	 	: OUT	STD_LOGIC
   );
END Timer_Datapath ;

ARCHITECTURE schematic OF Timer_Datapath IS  
-- Components
	COMPONENT Octuple_MUX4 IS  
		Port (		-- 8 bits per channel, 
			Ch0, Ch1	: 	IN 	STD_LOGIC_VECTOR (7 downto 0);  
			Ch2, Ch3	: 	IN 	STD_LOGIC_VECTOR (7 downto 0); 
			S			:	IN 	STD_LOGIC_VECTOR (1 downto 0);  -- 4 channels
			E			: 	IN 	STD_LOGIC; 			-- enable input
			Y		   	: 	OUT STD_LOGIC_VECTOR (7 downto 0)
		);
	END COMPONENT;
	
	COMPONENT data_register_8bit IS  
		Port (	CLK 	: 	IN	STD_LOGIC;
					CD		: 	IN	STD_LOGIC;
					LD 	  	: 	IN	STD_LOGIC;
					Q 	  	: 	OUT	STD_LOGIC_VECTOR(7 DOWNTO 0);
					D_in  	: 	IN STD_LOGIC_VECTOR(7 DOWNTO 0)
		);
	END COMPONENT;

	COMPONENT counter_8bit IS  
		Port (	CLK 	  		: IN	STD_LOGIC;
				CD 	  			: IN	STD_LOGIC;
				LD, CE, UD_L	: IN	STD_LOGIC;			
				D_in  			: IN	STD_LOGIC_VECTOR(7 DOWNTO 0);
				Q	  			: OUT	STD_LOGIC_VECTOR(7 DOWNTO 0);
				TC256  			: OUT	STD_LOGIC
		);       
	END COMPONENT;
	
	COMPONENT Adder_8bit IS  
	   port
	   (
		  A, B 		: std_logic_vector(7 downto 0); 
		  Cin		: in std_logic;		
		  S       	: out std_logic_vector(7 downto 0);
		  Z 		: out std_logic;
		  Cout 		: out std_logic
	   );
	END COMPONENT;
	
-- Signals for connecting components together (just the internal wires)  
	SIGNAL T1R, T2R, T3R, T4R	: STD_LOGIC_VECTOR (7 downto 0); 
	SIGNAL Y, Timer_value		: STD_LOGIC_VECTOR (7 downto 0); 
	SIGNAL Sel_time_R			: STD_LOGIC_VECTOR (1 downto 0); 		

-- Component pins not used
	SIGNAL Q		: std_logic_vector(7 downto 0);	
	SIGNAL K		: std_logic_vector(5 downto 0);	
	SIGNAL Cout, Z	: std_logic;		
	
	BEGIN  


-- Instantiation of components
	Chip1 : data_register_8bit        
-- 	from component name => to signal or port name	
		PORT MAP (
			CLK			=> CLK,
			CD			=> CD,
			LD			=> LD_time,
			D_in		=> T1,
			Q			=> T1R
		);

	Chip2 : data_register_8bit        
-- 	from component name => to signal or port name	
		PORT MAP (
			CLK			=> CLK,
			CD			=> CD,
			LD			=> LD_time,
			D_in		=> T2,
			Q			=> T2R
		);
		
	Chip3 : data_register_8bit        
-- 	from component name => to signal or port name	
		PORT MAP (
			CLK			=> CLK,
			CD			=> CD,
			LD			=> LD_time,
			D_in		=> T3,
			Q			=> T3R
		);
	
	Chip4 : data_register_8bit    
-- 	from component name => to signal or port name	
		PORT MAP (
			CLK			=> CLK,
			CD			=> CD,
			LD			=> LD_time,
			D_in		=> T4,
			Q			=> T4R
		);
	-- To show how to use only a section or a larger chip
	Chip5 : data_register_8bit    
-- 	from component name => to signal or port name	
		PORT MAP (
			CLK				=> CLK,
			CD				=> CD,
			LD				=> LD_time,
			D_in(7 downto 2)=> "000000",
			D_in(1 downto 0)=> Sel_Time,
			Q(7 downto 2)	=> K, -- This 6 D_FF are not used
			Q(1 downto 0)	=> Sel_Time_R -- The registered 2 bits
		);

	Chip6 : Octuple_MUX4    
-- 	from component name => to signal or port name	
		PORT MAP (
			Ch0	=> T1R,
			Ch1	=> T2R,
			Ch2	=> T3R,
			Ch3	=> T4R,
			S	=> Sel_Time_R,
			E	=> '1', 
			Y	=> Y
		);

	Chip7 : Adder_8bit    -- Y = A + B = A + (-4)
-- 	from component name => to signal or port name	
		PORT MAP (
			A		=> Y,
			B		=> "11111100", -- This is the (-4) using 2C 8-bit       
			Cin		=> '0',	      
			S		=> Timer_value,	      
			Z		=> Z,	      
			Cout	=> Cout
		);
		
	Chip8 : Counter_8bit
-- 	from component name => to signal or port name	
		PORT MAP (
			CLK	=> TCLK,
			CD	=> CD,
			LD	=> LD_Counter,     
			CE	=> CNT_enable,    
			UD_L	=> '0', -- Always down-counting    
			D_in 	=> Timer_value, 
			Q	=> Q,     
			TC256	=> Terminal_Count
		);
		
		
	-- Other connections and logic between components

END schematic ;
