--------------------------------------------------------------------------------
-- EETAC - CSD - P8 - http://digsys.upc.edu
--------------------------------------------------------------------------------
-- The top schematic of the programmable timer.
-- Strategy: plan C2. Hierarchical structure of components, logic and signals
-- The translation of the sketch in https://digsys.upc.edu/csd/exams/D2_21/D2_21.html
--------------------------------------------------------------------------------

LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
	
ENTITY programmable_timer	IS  
   Port (
	CLK, CD		: IN	STD_LOGIC;
	TCLK	    	: IN	STD_LOGIC;
	Trigger		: IN	STD_LOGIC;
	T1, T2, T3, T4	: IN	STD_LOGIC_VECTOR (7 downto 0);  
	Sel_Time	: IN	STD_LOGIC_VECTOR (1 downto 0);        
	Timer_OUT	: OUT	STD_LOGIC;     
	Timer_FLAG	: OUT	STD_LOGIC
   );
END programmable_timer ;

ARCHITECTURE schematic OF programmable_timer IS  
-- Components

	COMPONENT Timer_FSM IS  
		Port (
	      CLK, CD       		: IN	STD_LOGIC;
	      Trigger	    		: IN	STD_LOGIC;  
	      Terminal_Count		: IN	STD_LOGIC;      
	      LD_time, LD_Counter 	: OUT	STD_LOGIC; 
	      CNT_enable		: OUT	STD_LOGIC; 
	      Timer_OUT, Timer_Flag	: OUT	STD_LOGIC
		);
	END COMPONENT;

	COMPONENT Timer_datapath IS  
	   Port (
	      CLK, CD       		: IN	STD_LOGIC;
	      TCLK			: IN	STD_LOGIC;
	      LD_time, LD_Counter   	: IN	STD_LOGIC;  
	      CNT_enable	   	: IN	STD_LOGIC; 
	      T1, T2, T3, T4 		: IN	STD_LOGIC_VECTOR (7 downto 0);  
	      Sel_Time			: IN	STD_LOGIC_VECTOR (1 downto 0);  
	      Terminal_Count	 	: OUT	STD_LOGIC
	   );
	END COMPONENT;
       
-- Control signals from the FSM to the datapath: 
	SIGNAL LD_time, LD_Counter	: std_logic;
	SIGNAL CNT_enable		: std_logic;

-- This is the status signal from the datapath to the control unit
	SIGNAL Terminal_count		: std_logic; 

	BEGIN  
-- Instantiation of components
	Chip1 : Timer_FSM        
-- 	from component name 	=> to signal or port name	
		PORT MAP (
			CLK			=> CLK,
			CD			=> CD,
			Trigger			=> Trigger,
			Terminal_Count		=> Terminal_Count,
			LD_time			=> LD_time,
			LD_Counter		=> LD_Counter,
			CNT_enable		=> CNT_enable,
 			Timer_OUT		=> Timer_OUT,
 			Timer_Flag		=> Timer_Flag
		);

	Chip2	: Timer_datapath        
-- 	from component name 	=> to signal or port name	
		PORT MAP (
			TCLK			=> TCLK,
			CLK			=> CLK,
			CD 			=> CD,
			LD_time 	   	=> LD_time,
			LD_Counter    		=> LD_Counter,
			CNT_enable		=> CNT_enable,
 			Sel_Time    		=> Sel_Time,								
			T1    			=> T1,
			T2    			=> T2,
			T3     			=> T3,
			T4    			=> T4,
			Terminal_Count  	=> Terminal_Count
		);
END schematic ;
